@@ -800,8 +800,8 @@ func (s *SourceState) Read(ctx context.Context, options *ReadOptions) error {
 	allSourceStateEntries := make(map[RelPath][]SourceStateEntry)
 	addSourceStateEntries := func(relPath RelPath, sourceStateEntries ...SourceStateEntry) {
 		allSourceStateEntriesMu.Lock()
+		defer allSourceStateEntriesMu.Unlock()
 		allSourceStateEntries[relPath] = append(allSourceStateEntries[relPath], sourceStateEntries...)
-		allSourceStateEntriesMu.Unlock()
 	}
 	walkFunc := func(sourceAbsPath AbsPath, fileInfo fs.FileInfo, err error) error {
 		if err != nil {
@@ -876,6 +876,18 @@ func (s *SourceState) Read(ctx context.Context, options *ReadOptions) error {
 			}
 			sourceStateDir := s.newSourceStateDir(sourceAbsPath, sourceRelPath, da)
 			addSourceStateEntries(targetRelPath, sourceStateDir)
+			if da.External {
+				sourceStateEntries, err := s.readExternalDir(sourceAbsPath, sourceRelPath, targetRelPath)
+				if err != nil {
+					return err
+				}
+				allSourceStateEntriesMu.Lock()
+				for relPath, entries := range sourceStateEntries {
+					allSourceStateEntries[relPath] = append(allSourceStateEntries[relPath], entries...)
+				}
+				allSourceStateEntriesMu.Unlock()
+				return fs.SkipDir
+			}
 			if sourceStateDir.Attr.Remove {
 				s.Lock()
 				s.removeDirs[targetRelPath] = struct{}{}
@@ -1794,11 +1806,9 @@ func (s *SourceState) newSourceStateFile(
 }
 
 // newSourceStateDirEntry returns a SourceStateEntry constructed from a directory in s.
-//
-// We return a SourceStateEntry rather than a *SourceStateDir to simplify nil checks later.
 func (s *SourceState) newSourceStateDirEntry(
 	actualStateDir *ActualStateDir, fileInfo fs.FileInfo, parentSourceRelPath SourceRelPath, options *AddOptions,
-) (SourceStateEntry, error) {
+) *SourceStateDir {
 	dirAttr := DirAttr{
 		TargetName: fileInfo.Name(),
 		Exact:      options.Exact,
@@ -1813,17 +1823,14 @@ func (s *SourceState) newSourceStateDirEntry(
 		targetStateEntry: &TargetStateDir{
 			perm: fs.ModePerm &^ s.umask,
 		},
-	}, nil
+	}
 }
 
 // newSourceStateFileEntryFromFile returns a SourceStateEntry constructed from a
 // file in s.
-//
-// We return a SourceStateEntry rather than a *SourceStateFile to simplify nil
-// checks later.
 func (s *SourceState) newSourceStateFileEntryFromFile(
 	actualStateFile *ActualStateFile, fileInfo fs.FileInfo, parentSourceRelPath SourceRelPath, options *AddOptions,
-) (SourceStateEntry, error) {
+) (*SourceStateFile, error) {
 	fileAttr := FileAttr{
 		TargetName: fileInfo.Name(),
 		Encrypted:  options.Encrypt,
@@ -1874,13 +1881,10 @@ func (s *SourceState) newSourceStateFileEntryFromFile(
 
 // newSourceStateFileEntryFromSymlink returns a SourceStateEntry constructed
 // from a symlink in s.
-//
-// We return a SourceStateEntry rather than a *SourceStateFile to simplify nil
-// checks later.
 func (s *SourceState) newSourceStateFileEntryFromSymlink(
 	actualStateSymlink *ActualStateSymlink, fileInfo fs.FileInfo, parentSourceRelPath SourceRelPath,
 	options *AddOptions,
-) (SourceStateEntry, error) {
+) (*SourceStateFile, error) {
 	linkname, err := actualStateSymlink.Linkname()
 	if err != nil {
 		return nil, err
@@ -2108,6 +2112,98 @@ func (s *SourceState) readExternalArchive(
 	return sourceStateEntries, nil
 }
 
+// ReadExternalDir returns all source state entries in an external_ dir.
+func (s *SourceState) readExternalDir(
+	rootSourceAbsPath AbsPath, rootSourceRelPath SourceRelPath, rootTargetRelPath RelPath,
+) (map[RelPath][]SourceStateEntry, error) {
+	sourceStateEntries := make(map[RelPath][]SourceStateEntry)
+	walkFunc := func(absPath AbsPath, fileInfo fs.FileInfo, err error) error {
+		switch {
+		case err != nil:
+			return err
+		case absPath == rootSourceAbsPath:
+			return nil
+		}
+		relPath := absPath.MustTrimDirPrefix(rootSourceAbsPath)
+		targetRelPath := rootTargetRelPath.Join(relPath)
+		if s.Ignore(targetRelPath) {
+			if fileInfo.IsDir() {
+				return fs.SkipDir
+			}
+			return nil
+		}
+		var sourceStateEntry SourceStateEntry
+		switch fileInfo.Mode().Type() {
+		case 0:
+			fileAttr := FileAttr{
+				TargetName: fileInfo.Name(),
+				Type:       SourceFileTypeFile,
+				Empty:      true,
+				Private:    isPrivate(fileInfo),
+				ReadOnly:   isReadOnly(fileInfo),
+			}
+			lazyContents := newLazyContentsFunc(func() ([]byte, error) {
+				return s.system.ReadFile(absPath)
+			})
+			sourceStateEntry = &SourceStateFile{
+				lazyContents:  lazyContents,
+				origin:        SourceStateOriginAbsPath(absPath),
+				Attr:          fileAttr,
+				sourceRelPath: rootSourceRelPath.Join(relPath.SourceRelPath()),
+				targetStateEntry: &TargetStateFile{
+					lazyContents: lazyContents,
+					empty:        true,
+					perm:         fileAttr.perm() &^ s.umask,
+				},
+			}
+		case fs.ModeDir:
+			dirAttr := DirAttr{
+				TargetName: fileInfo.Name(),
+				Exact:      true,
+				Private:    isPrivate(fileInfo),
+				ReadOnly:   isReadOnly(fileInfo),
+			}
+			sourceStateEntry = &SourceStateDir{
+				origin:        SourceStateOriginAbsPath(absPath),
+				sourceRelPath: rootSourceRelPath.Join(relPath.SourceRelDirPath()),
+				Attr:          dirAttr,
+				targetStateEntry: &TargetStateDir{
+					perm: dirAttr.perm() &^ s.umask,
+				},
+			}
+		case fs.ModeSymlink:
+			fileAttr := FileAttr{
+				TargetName: fileInfo.Name(),
+				Type:       SourceFileTypeFile,
+			}
+			lazyLinkname := newLazyLinknameFunc(func() (string, error) {
+				return s.system.Readlink(absPath)
+			})
+			sourceStateEntry = &SourceStateFile{
+				lazyContents: newLazyContentsFunc(func() ([]byte, error) {
+					linkname, err := lazyLinkname.Linkname()
+					if err != nil {
+						return nil, err
+					}
+					return []byte(linkname), nil
+				}),
+				origin:        SourceStateOriginAbsPath(absPath),
+				Attr:          fileAttr,
+				sourceRelPath: rootSourceRelPath.Join(relPath.SourceRelPath()),
+				targetStateEntry: &TargetStateSymlink{
+					lazyLinkname: lazyLinkname,
+				},
+			}
+		}
+		sourceStateEntries[targetRelPath] = append(sourceStateEntries[targetRelPath], sourceStateEntry)
+		return nil
+	}
+	if err := Walk(s.system, rootSourceAbsPath, walkFunc); err != nil {
+		return nil, err
+	}
+	return sourceStateEntries, nil
+}
+
 // readExternalFile reads an external file and returns its SourceStateEntries.
 func (s *SourceState) readExternalFile(
 	ctx context.Context, externalRelPath RelPath, parentSourceRelPath SourceRelPath, external *External,
@@ -2245,7 +2341,7 @@ func (s *SourceState) sourceStateEntry(
 	case *ActualStateAbsent:
 		return nil, fmt.Errorf("%s: not found", destAbsPath)
 	case *ActualStateDir:
-		return s.newSourceStateDirEntry(actualStateEntry, fileInfo, parentSourceRelPath, options)
+		return s.newSourceStateDirEntry(actualStateEntry, fileInfo, parentSourceRelPath, options), nil
 	case *ActualStateFile:
 		return s.newSourceStateFileEntryFromFile(actualStateEntry, fileInfo, parentSourceRelPath, options)
 	case *ActualStateSymlink:
