@@ -61,8 +61,13 @@ const (
 	logComponentValueSystem          = "system"
 )
 
-type purgeOptions struct {
-	binary bool
+type doPurgeOptions struct {
+	binary          bool
+	cache           bool
+	config          bool
+	persistentState bool
+	sourceDir       bool
+	workingTree     bool
 }
 
 type templateConfig struct {
@@ -198,6 +203,7 @@ type Config struct {
 	sourceState         *chezmoi.SourceState
 	sourceStateErr      error
 	templateData        *templateData
+	runEnv              []string
 
 	stdin       io.Reader
 	stdout      io.Writer
@@ -1863,6 +1869,7 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 
 	scriptEnv := os.Environ()
 	templateData := c.getTemplateData()
+	scriptEnv = append(scriptEnv, "CHEZMOI=1")
 	for key, value := range map[string]string{
 		"ARCH":          templateData.Arch,
 		"ARGS":          strings.Join(templateData.Args, " "),
@@ -1897,6 +1904,7 @@ func (c *Config) persistentPreRunRootE(cmd *cobra.Command, args []string) error
 	for key, value := range c.ScriptEnv {
 		scriptEnv = append(scriptEnv, key+"="+value)
 	}
+	c.runEnv = scriptEnv
 	realSystem.SetScriptEnv(scriptEnv)
 
 	return nil
@@ -2072,6 +2080,7 @@ func (c *Config) run(dir chezmoi.AbsPath, name string, args []string) error {
 		}
 		cmd.Dir = dirRawAbsPath.String()
 	}
+	cmd.Env = c.runEnv
 	cmd.Stdin = c.stdin
 	cmd.Stdout = c.stdout
 	cmd.Stderr = c.stderr
